'use strict';

angular.module('coffeeorderingonline', ['coffeeorderingonline.controllers', 'coffeeorderingonline.services', 'coffeeorderingonline.directives', 'coffeeorderingonline.filters', 'ui.select', 'ngSanitize'
            , 'ngAnimate', 'ui.bootstrap', 'ui.router', 'ui.grid', 'ui.grid.edit', 'ui.grid.selection', 'ng-context-menu', 'bsLoadingOverlay', 'dndLists',
            'ui.grid.draggable-rows', 'google.places', , 'ui.grid.exporter', 'ngCsvImport','ui.grid.pagination' /*, 'angularjs-dropdown-multiselect' /*'ngAutocomplete'*/])

        .run(function ($rootScope, Configuration, $state, $log, bsLoadingOverlayService) {
            $rootScope.$state = $state;
            Configuration.initialize();
            $rootScope.$on('$stateChangeStart', function (event, toState, toParams) {

                var requireLogin = toState.data.requireLogin;
                var token = Configuration.getConfiguration().jsonwebtoken;
                //$log.info("token = " + token );

                if (requireLogin && (token === '' || token === undefined)) {
                    //$log.info('preventDefault');
                    event.preventDefault();
                    // get me a login modal!

                    $state.go('login');
                }
            });
            bsLoadingOverlayService.setGlobalConfig({
                delay: 500,
                templateUrl: 'templates/overlay/loading-overlay.html' // Template url for overlay element. If not specified - no overlay element is created.

            });

        }).config(function ($stateProvider, $urlRouterProvider, $httpProvider) {

            
          $stateProvider
            .state('login', {
                url: '/login',
                templateUrl: "templates/login/login.html",
                controller: "LoginCtrl",
                data: {
                    requireLogin: false
                }
            })
            .state('base', {
                abstract: true,
                template: "<ui-view/>",
                data: {
                    requireLogin: true // this property will apply to all children of 'app'
                }
            })
            .state('base.home', {
                url: '/home',
                templateUrl: "templates/home/mainpage.html",
                controller: "HomeCtrl",
                data: {
                    requireLogin: true
                    //bodyClass: 'homeimage'
                }
            })
            .state('base.createorder', {
                url: '/createorder',
                templateUrl: "templates/createorder/createorder.html",
                controller: "CreateOrderCtrl",
                data: {
                    requireLogin: true
                }
            })
            .state('base.cancelorder', {
                url: '/cancelorder',
                templateUrl: "templates/cancelorder/cancelorder.html",
                controller: "CancelOrderCtrl",
                data: {
                    requireLogin: true
                }
            })
            .state('base.transferpatient', {
                url: '/transferpatient',
                templateUrl: 'templates/transferpatient/transferpatient.html',
                controller: 'TransferPatientCtrl',
                data:{
                    requireLogin: true
                }
            })
            .state('base.admin', {
                url: '/admin',
                templateUrl: "templates/admin/admin.html",
                controller: "AdminCtrl",
                data: {
                    requireLogin: true
                }
            })
    // if none of the above states are matched, use this as the fallback
    $urlRouterProvider.otherwise('/login');
    // this interceptor handle adding authorization header with jwt token 
    // and also re authentication after a 401
    $httpProvider.interceptors.push('AuthInterceptor');



}).config(function ($provide) {
    $provide.decorator("$q", function ($delegate) {
        //Helper method copied from q.js.
        var isPromiseLike = function (obj) {
            return obj && angular.isFunction(obj.then);
        }

        /*
         * @description Execute a collection of tasks serially.  A task is a function that returns a promise
         *
         * @param {Array.<Function>|Object.<Function>} tasks An array or hash of tasks.  A tasks is a function
         *   that returns a promise.  You can also provide a collection of objects with a success tasks, failure task, and/or notify function
         * @returns {Promise} Returns a single promise that will be resolved or rejected when the last task
         *   has been resolved or rejected.
         */
        function serial(tasks) {
            //Fake a "previous task" for our initial iteration
            var prevPromise;
            var error = new Error();
            angular.forEach(tasks, function (task, key) {
                var success = task.success || task;
                var fail = task.fail;
                var notify = task.notify;
                var nextPromise;

                //First task
                if (!prevPromise) {
                    nextPromise = success();
                    if (!isPromiseLike(nextPromise)) {
                        error.message = "Task " + key + " did not return a promise.";
                        throw error;
                    }
                } else {
                    //Wait until the previous promise has resolved or rejected to execute the next task
                    nextPromise = prevPromise.then(
                            /*success*/function (data) {
                                if (!success) {
                                    return data;
                                }
                                var ret = success(data);
                                if (!isPromiseLike(ret)) {
                                    error.message = "Task " + key + " did not return a promise.";
                                    throw error;
                                }
                                return ret;
                            },
                            /*failure*/function (reason) {
                                if (!fail) {
                                    return $delegate.reject(reason);
                                }
                                var ret = fail(reason);
                                if (!isPromiseLike(ret)) {
                                    error.message = "Fail for task " + key + " did not return a promise.";
                                    throw error;
                                }
                                return ret;
                            },
                            notify);
                }
                prevPromise = nextPromise;
            });

            return prevPromise || $delegate.when();
        }

        $delegate.serial = serial;
        return $delegate;
    });
});

var orderingControllers = angular.module('coffeeorderingonline.controllers', []);
var orderingDirectives = angular.module('coffeeorderingonline.directives', []);
var orderingServices = angular.module('coffeeorderingonline.services', ['ng']);
var orderingFilters = angular.module('coffeeorderingonline.filters', []);

orderingControllers.controller('NavCtrl', function ($scope, $location, $state,Configuration,$window,CustomerResource) {
        $scope.configuration = Configuration.getConfiguration();
    
        $scope.operatorName = $scope.configuration.operatorName;
//        $scope.combinedCustomerBean = $scope.configuration.combinedCustomerBean;

        $scope.showNav = true;
        
        if( sessionStorage.customers !== null ){
            $scope.customers = angular.fromJson( sessionStorage.customers );
        }else{
            $scope.customers = [];
        }
        $scope.loggedInUser = {};
        $scope.loggedInUser.accountType = sessionStorage.accountType;
     
        console.log("Window");
        console.log("Contains home = " + $window.location.href.includes("#/home") );
     
        $scope.enableCombos = $window.location.href.includes("#/home");
     
     
        $scope.loadCombos = function() {
            console.log("nav ctrl scope");
            console.log($scope.configuration);
            
               
            $scope.loggedInUser.accountType = sessionStorage.accountType;
            
            if( $scope.loggedInUser.accountType === 'S' ){
                CustomerResource.getCorsCombinedCustomerBeans().then(function(customers){
                    $scope.customers = customers;
                    console.log("$scope.customers");
                    console.log($scope.customers);
                    
                    if( $scope.configuration.combinedCustomerBean.id === null ){
                        $scope.configuration.combinedCustomerBean = $scope.customers[0];
                        if( $scope.configuration.combinedCustomerBean.contacts.length > 0 ){
                            $scope.configuration.selectedContact = $scope.configuration.combinedCustomerBean.contacts[0];
                            console.log("SelectedContact = ")
                            console.log( $scope.configuration.selectedContact );
                        };
                        Configuration.saveLoginConfiguration();
                    };
                    
                    sessionStorage.customers = angular.toJson( customers );
                });   
            }
        
        };
        
        
        
        $scope.navClass = function (page) {
          var currentRoute = $location.path().substring(1) || 'home';
          if(currentRoute === "login" )
                  $scope.showNav = false;
          else
                  $scope.showNav = true;
          return page === currentRoute ? 'active' : '';
        };
  
        $scope.logout = function () {
            
            console.log("im running");
            
            sessionStorage.jsonwebtoken="";
            sessionStorage.username="";
            sessionStorage.password="";
            sessionStorage.accountType = "";
            sessionStorage.selectedContact = "";
            $scope.loggedInUser.accountType = "";
            Configuration.clearLoginConfiguration();
            $state.go('login');
        };
        
        $scope.customerChange = function(){
            Configuration.saveLoginConfiguration();
            console.log( "customer changed");
            console.log( $scope.configuration.combinedCustomerBean.contacts );
            // set the contact 
            if( $scope.configuration.combinedCustomerBean.contacts.length > 0 ){
                $scope.configuration.selectedContact = $scope.configuration.combinedCustomerBean.contacts[0];
                console.log("SelectedContact = ")
                console.log( $scope.configuration.selectedContact );
            };
            Configuration.saveLoginConfiguration();
        };
        
        $scope.contactChange = function(){
             Configuration.saveLoginConfiguration();
        };
        
        $scope.loadCombos();
 });


orderingDirectives.directive('ngRightClick', function ($parse) {
    return function (scope, element, attrs) {
        var fn = $parse(attrs.ngRightClick);
        element.bind('contextmenu', function (event) {
            scope.$apply(function () {
                event.preventDefault();
                fn(scope, {$event: event});
            });
        });
    };
});

orderingFilters.filter('split', function ($log) {
    return function (input, splitChar, splitIndex) {
        // do some bounds checking here to ensure it has that index
        $log.info(input);
        if (input === undefined || input === null)
            return input;

        return input.split(splitChar)[splitIndex];
    };
});